
import gtk
import gtk.glade
import DSNodeTypes
from Globals import *

from pprint import pprint
import string, random

# This is probably complete crack, but I'm trying to separate the 
# "Should this tab be shown" logic from the properties dialog, and
# it doesn't make sense to instantiate the class if it's inappropriate
# for the node... So, we just pass the node to each of the "TabFactories"
# and they decide whether the tab gets added or not.
TabFactories = []

_NodePropertiesDialogs = {}

def showNodePropertiesDialog(node):
    if _NodePropertiesDialogs.has_key(node.Path()):
        _NodePropertiesDialogs[node.Path()].Show()
    else:
        _NodePropertiesDialogs[node.Path()] = NodePropertiesDialog(node)

def closeNodePropertiesDialog(node):
    if _NodePropertiesDialogs.has_key(node.Path()):
        _NodePropertiesDialogs[node.Path()].Close()

def closeAllNodeDialogs():
    keys = _NodePropertiesDialogs.keys()
    for key in keys:
        print _NodePropertiesDialogs
        dlg = _NodePropertiesDialogs[key]
        dlg.Close()

class NodePropertiesDialog:
    
    def __init__(self, node):
        self.node = node

        self._registered_path = self.node.Path()

        self.gxml = gtk.glade.XML(GLADE_FILE, 'dialog1')
        self.dlg = self.gxml.get_widget('dialog1')

        self.dlg.set_title("'%s' - Properties" % (self.node.Name()))

        # Connect the glade signals
        function_dict = dict([(f, getattr(self, f)) for f in dir(self)]) 
        self.gxml.signal_autoconnect(function_dict)

        # Collect the appropriate tabs for this node
        global TabFactories
        self.tabs = []
        for tabType in TabFactories:
            try:
                tab = tabType(self.node, self)
                if (tab):
                    self.tabs.append(tab)
            except Exception, err:
                # Don't let one bad tab prevent the others from loading
                print "Exception: %s" % (err)


    def appendPage(self, widget, label):
        return self.gxml.get_widget('notebook1').append_page(widget, gtk.Label(label))
        self.gxml.get_widget('notebook1').set_show_tabs(True)

    def on_ok_clicked(self, obj, *args):

                for tab in self.tabs:
                    tab.Apply()

                self.node.SetInfo()

                self.Unregister()
                self.dlg.hide()
                self.dlg.destroy()

    def on_cancel_clicked(self, obj, *args):
        self.Close()

    def Unregister(self):
        global _NodePropertiesDialogs
        # The key could be deleted through either Close() or
        # the dialog delete event. Too lazy to make there only
        # be one way at the moment. Just check before deleting.
        if (_NodePropertiesDialogs.has_key(self._registered_path)):
            del _NodePropertiesDialogs[self._registered_path]
       
    def Show(self):
        self.dlg.show_all()
        self.dlg.grab_focus()

    def Close(self):
        # Reset the node properties
        try:
            self.node.GetInfo()
        except:
            pass
        for tab in self.tabs:
            tab.Close()
        self.Unregister()
        self.dlg.destroy()

    def on_dialog_delete_event(self, obj, *args):
        self.Unregister()
        # returning False lets the dialog get destroyed
        return False

###
### Tab factories
###

# Tab for generic user account properties

class UserAccountTab:
    fieldNames = ("FirstName", "Initials", "LastName", "DisplayName",
                  "Description", "Office", "TelephoneNumber", "EmailAddress")

    def __init__(self, propertiesDialog):
        # Snip out the part of the dialog that we want
        # and add it as a tab
        self.node = propertiesDialog.node

        self.gxml = gtk.glade.XML(GLADE_FILE, 'userAccount')
        dlg = self.gxml.get_widget('dialog-vbox2')
        dlgcontents = self.gxml.get_widget('dlgcontents')
        dlg.remove(dlgcontents)
        dlg.hide()
        dlg.destroy()
        propertiesDialog.appendPage(dlgcontents, "General")
        # Connect the glade signals
        function_dict = dict([(f, getattr(self, f)) for f in dir(self)]) 
        self.gxml.signal_autoconnect(function_dict)

        self.password_dlg = None
        self.initFields()

    def initFields(self):
        for f in self.fieldNames:
            self.gxml.get_widget(f).set_text(self.node[f])

        self.gxml.get_widget('DisplayNameLabel').set_text(self.node['DisplayName'])
        self.gxml.get_widget('uid').set_text(self.node['Uid'])

    def OnPasswordButtonClicked(self, button, *args):
        self.password_dlg = PasswordChangeDialog(self.node)
        self.password_dlg.run()
        self.password_dlg.destroy()
        self.password_dlg = None
        
    def _widgetsToDict(self):
        d = dict(self.original_values)
        for f in self.stringFields:
            value = self.gxml.get_widget(f).get_text()
            if (value):
                d[f] = [self.gxml.get_widget(f).get_text()]
            else:
                d[f] = ''
        return d

    def Apply(self):
        """Apply any changes that have been made through this tab"""
        for f in self.fieldNames:
            value = self.gxml.get_widget(f).get_text()
            self.node[f] = self.gxml.get_widget(f).get_text()
       
    def Close(self):
        if self.password_dlg:
            self.password_dlg.destroy()

class PasswordChangeDialog:
    
    def __init__(self, node):
        self.node = node
        self.gxml = gtk.glade.XML(GLADE_FILE, 'changePasswordDialog')
        self.dlg = self.gxml.get_widget('changePasswordDialog')
        self.generateButton = self.gxml.get_widget('generateButton')
        self.generatedPassword = self.gxml.get_widget('generatedPassword')
        
        self.manualPassRadioButton = self.gxml.get_widget("manualPassRadioButton")

        self.timeout_id = None
        # Connect the glade signals
        function_dict = dict([(f, getattr(self, f)) for f in dir(self)]) 
        self.gxml.signal_autoconnect(function_dict)

    
    def run(self):
        password = None
        res = self.dlg.run()
        if (self.timeout_id):
            gtk.timeout_remove(self.timeout_id)
            self.timeout_id = None
        
        self.dlg.hide()
        if (res == gtk.RESPONSE_OK):
            if (self.manualPassRadioButton.get_active()):
                password1 = self.gxml.get_widget('password1').get_text()
                password2 = self.gxml.get_widget('password2').get_text()
                if not (password1 == password2):
                    print "FIXME: passwords don't match"
                else:
                    password = password1
            else:
                password = self.generatedPassword.get_text()
            
            if (password):
                self.node.SetPassword(password)
        self.dlg.destroy()

    def destroy(self):
        self.dlg.hide()
        self.dlg.destroy()

    def OnPasswordChanged(self, obj, *args):
        password1 = self.gxml.get_widget('password1')
        password2 = self.gxml.get_widget('password2')
        okbutton = self.gxml.get_widget('okbutton5')
        passwordsDoNotMatch = self.gxml.get_widget('passwordsDoNotMatch')

        if (self.timeout_id):
            gtk.timeout_remove(self.timeout_id)
            self.timeout_id = None

        if (password1.get_text() == password2.get_text()):
            passwordsDoNotMatch.hide()
            okbutton.set_sensitive(True)
        else:
            if (password2.get_text()):
                self.timeout_id = gtk.timeout_add(2000, self.PasswordTimeout)
            else:
                self.timeout_id = gtk.timeout_add(6000, self.PasswordTimeout)
            okbutton.set_sensitive(False)

    def PasswordTimeout(self):
        password1 = self.gxml.get_widget('password1')
        password2 = self.gxml.get_widget('password2')
        passwordsDoNotMatch = self.gxml.get_widget('passwordsDoNotMatch')

        if not (password1.get_text() == password2.get_text()):
            passwordsDoNotMatch.show()
        
        return False

    def OnManualPassToggled(self, obj):
        password1 = self.gxml.get_widget('password1')
        password2 = self.gxml.get_widget('password2')
        okbutton = self.gxml.get_widget('okbutton5')
        passwordsDoNotMatch = self.gxml.get_widget('passwordsDoNotMatch')

        if (self.manualPassRadioButton.get_active()):
            password1.set_sensitive(True)
            password2.set_sensitive(True)
            self.generateButton.set_sensitive(False)
            self.generatedPassword.set_sensitive(False)
            if (password1.get_text()):
                if (password1.get_text() == password2.get_text()):
                    okbutton.set_sensitive(True)
                    passwordsDoNotMatch.hide()
                else:
                    okbutton.set_sensitive(False)
                    passwordsDoNotMatch.show()
            else:
                okbutton.set_sensitive(False)
                passwordsDoNotMatch.hide()
                    
        else:
            if (self.timeout_id):
                gtk.timeout_remove(self.timeout_id)
                self.timeout_id = None
            
            password1.set_sensitive(False)
            password2.set_sensitive(False)
            self.generateButton.set_sensitive(True)
            self.generatedPassword.set_sensitive(True)
            self.GeneratePassword(None)
            okbutton.set_sensitive(True)
            passwordsDoNotMatch.hide()
        

    def GeneratePassword(self, obj):
        allchars = string.ascii_letters + string.digits
        newpass = []
        for i in range(6):
            newpass.append(allchars[random.randint(0, len(allchars) - 1)])
        self.generatedPassword.set_text(''.join(newpass))


def UserAccountTabFactory(node, propertiesDialog):
    if (node.Implements('IDSUser')):
        return UserAccountTab(propertiesDialog)
    else:
        return None

TabFactories.append(UserAccountTabFactory)


# Tab for viewing/editing raw ldap properties
class RawTab:
    def __init__(self, propertiesDialog):
        keypb = images['stock_edit']
        
        page = gtk.ScrolledWindow()
        page.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        treeview = gtk.TreeView()
        page.add(treeview)
        page.show_all()
        propertiesDialog.appendPage(page, "Raw Properties")

        tableModel = gtk.TreeStore(gtk.gdk.Pixbuf, str, str)
        cellpb = gtk.CellRendererPixbuf()
        cellpb.set_property('xalign', 0.0)
        celltext = gtk.CellRendererText()
        col1 = gtk.TreeViewColumn('Name', cellpb, pixbuf=0)
        col1.pack_start(celltext, False)
        col1.add_attribute(celltext, 'text', 1)
        celltext = gtk.CellRendererText()
        col2 = gtk.TreeViewColumn('Value', celltext, text=2)
        treeview.append_column(col1)
        treeview.append_column(col2)

        attributes = propertiesDialog.node._get_all()
        attribute_keys = attributes.keys()
        attribute_keys.sort()
        for key in attribute_keys:
            if (len(attributes[key]) == 1):
                tableModel.append(None, (keypb, key, attributes[key][0]))
                continue

            iter = tableModel.append(None, (keypb, key, ''))
            
            for val in attributes[key]:
                tableModel.append(iter, (None, None, val))

        treeview.set_model(tableModel)

    def Apply(self):
        """Apply any changes that have been made through this tab"""
        return

    def Close(self):
        pass

def RawTabFactory(node, propertiesDialog):
    
    if (RAW_MODE):
        return RawTab(propertiesDialog)
    else:
        return None

TabFactories.append(RawTabFactory)

class MemberOfTab:

    def __init__(self, propertiesDialog):
        # Snip out the part of the dialog that we want
        # and add it as a tab
        self.node = propertiesDialog.node

        self.gxml = gtk.glade.XML(GLADE_FILE, 'memberOf')
        dlg = self.gxml.get_widget('dialog-vbox3')
        dlgcontents = self.gxml.get_widget('dlgcontents')
        dlg.remove(dlgcontents)
        dlg.hide()
        dlg.destroy()
        propertiesDialog.appendPage(dlgcontents, "Member Of")
        # Connect the glade signals
        function_dict = dict([(f, getattr(self, f)) for f in dir(self)]) 
        self.gxml.signal_autoconnect(function_dict)

        self.initGroupList()

    def initGroupList(self):

        self.grouppb = images['stock_contact_list']
        self.userlistpb = images['stock_addressbook']

        self.treeview = self.gxml.get_widget('groupMembershipList')
        cellpb = gtk.CellRendererPixbuf()
        cellpb.set_property('xalign', 0.0)
        celltext = gtk.CellRendererText()
        col1 = gtk.TreeViewColumn('Name', cellpb, pixbuf=0)
        col1.pack_start(celltext, False)
        col1.add_attribute(celltext, 'text', 1)
        celltext = gtk.CellRendererText()
        col2 = gtk.TreeViewColumn('Path', celltext, text=2)
        celltoggle = gtk.CellRendererToggle()
        celltoggle.connect('toggled', self.OnPrimaryGroupToggled)
        celltoggle.set_radio(True)
        col3 = gtk.TreeViewColumn('Primary', celltoggle)
        col3.set_cell_data_func(celltoggle, self.PrimaryGroupCheck)
        self.treeview.append_column(col1)
        self.treeview.append_column(col2)
        self.treeview.append_column(col3)

        self.add_group_dlg = None
        
        self.refreshGroupList()

    def PrimaryGroupCheck(self, column, cell, model, iter, *args):
        groupNode = model.get_value(iter, 3)
        if (groupNode.Implements('IDSGroup')):
            cell.set_active(int(bool(self.node['PrimaryGroup']) and \
                        groupNode == self.node['PrimaryGroup']))
        else:
            cell.set_active(0)

    def refreshGroupList(self):
        groups = self.node.Groups()
        listModel = gtk.ListStore(gtk.gdk.Pixbuf, str, str, object)

        primaryGroup = self.node['PrimaryGroup']

        #if not (primaryGroup):
        #    self.gxml.get_widget('primaryGroup').set_text('UnknownGroup')
        if (primaryGroup):
            #self.gxml.get_widget('primaryGroup').set_text(primaryGroup.Name())

            if not primaryGroup in groups:
                groups.append(primaryGroup)
                # FIXME: should be using the listStore's sorting here
                groups.sort(lambda x, y: cmp(x.Name(), y.Name()))

        for group in groups:
            if group.Implements('IDSUserList'):
                pb = self.userlistpb
            else:
                pb = self.grouppb
            listModel.append((pb, group.Name(),
                    group.PrettyPath(), group))

        self.treeview.set_model(listModel)

    def OnPrimaryGroupToggled(self, celltoggle, path, *args):
        print "OnPrimaryGroupToggled", path
        model = self.treeview.get_model()
        iter = model.get_iter(path)
        groupNode = model.get_value(iter, 3)
        if groupNode.Implements('IDSGroup'):
            self.node['PrimaryGroup'] = groupNode

    def OnAddButtonClicked(self, obj, *args):
        gxml = gtk.glade.XML(GLADE_FILE, 'memberOf_AddGroup')
        self.add_group_dlg = gxml.get_widget('memberOf_AddGroup')
        treeview = gxml.get_widget('treeview2')
        selection = treeview.get_selection()
        selection.set_mode(gtk.SELECTION_MULTIPLE)
        
        listModel = gtk.ListStore(gtk.gdk.Pixbuf, str, str, object)
        cellpb = gtk.CellRendererPixbuf()
        cellpb.set_property('xalign', 0.0)
        celltext = gtk.CellRendererText()
        col1 = gtk.TreeViewColumn('Name', cellpb, pixbuf=0)
        col1.pack_start(celltext, False)
        col1.add_attribute(celltext, 'text', 1)
        celltext = gtk.CellRendererText()
        col2 = gtk.TreeViewColumn('Path', celltext, text=2)
        treeview.append_column(col1)
        treeview.append_column(col2)
       
        groups = self.node.Groups(invert=True)
        for group in groups:
            if group.Implements('IDSUserList'):
                pb = self.userlistpb
            else:
                pb = self.grouppb

            listModel.append((pb, group.Name(),
                    group.PrettyPath(), group))
        treeview.set_model(listModel)
        res = self.add_group_dlg.run()
        self.add_group_dlg.hide()
        if (res == gtk.RESPONSE_OK):
            selection = treeview.get_selection()
            treepaths = selection.get_selected_rows()[1]

            for treepath in treepaths:
                iter = listModel.get_iter(treepath)
                groupNode = listModel.get_value(iter, 3)
                groupNode.Add(self.node)
                groupNode.SetInfo()

            if (treepaths):
                self.refreshGroupList()

        self.add_group_dlg.destroy()
        self.add_group_dlg = None
       
    def OnRemoveButtonClicked(self, obj, *args):
        selection = self.treeview.get_selection()
        model, iter = selection.get_selected()
        groupNode = model.get_value(iter, 3)
        groupNode.Remove(self.node)
        groupNode.SetInfo()
        self.refreshGroupList()

    def Apply(self):
        """Apply any changes that have been made through this tab"""

        # the only change that gets made to this object directly is
        # primaryGroup. All other changes are done to the group objects
        # and are handled through the add/remove buttons
        
        return

    def Close(self):
        if self.add_group_dlg:
            self.add_group_dlg.destroy()

def MemberOfTabFactory(node, propertiesDialog):
    if (node.Implements('IDSUser')) or (node.Implements('IDSComputer')):
        return MemberOfTab(propertiesDialog)

TabFactories.append(MemberOfTabFactory)

class UserAdvancedTab:
    fieldNames = ("LoginShell", "HomeDirectory", "SambaLogonScript", "SambaHomePath",
                  "SambaHomeDrive")

    def __init__(self, propertiesDialog):
        # Snip out the part of the dialog that we want
        # and add it as a tab
        self.node = propertiesDialog.node

        self.gxml = gtk.glade.XML(GLADE_FILE, 'userAdvanced')
        dlg = self.gxml.get_widget('vbox18')
        dlgcontents = self.gxml.get_widget('dlgcontents')
        dlg.remove(dlgcontents)
        dlg.hide()
        dlg.destroy()
        propertiesDialog.appendPage(dlgcontents, "Advanced")
        # Connect the glade signals
        function_dict = dict([(f, getattr(self, f)) for f in dir(self)]) 
        self.gxml.signal_autoconnect(function_dict)

        self.initFields()

    def initFields(self):
        for f in self.fieldNames:
            self.gxml.get_widget(f).set_text(self.node[f])

    def _widgetsToDict(self):
        d = dict(self.original_values)
        for f in self.stringFields:
            value = self.gxml.get_widget(f).get_text()
            if (value):
                d[f] = [self.gxml.get_widget(f).get_text()]
            else:
                d[f] = ''
        return d

    def Apply(self):
        """Apply any changes that have been made through this tab"""
        for f in self.fieldNames:
            value = self.gxml.get_widget(f).get_text()
            self.node[f] = self.gxml.get_widget(f).get_text()

    def Close(self):
        pass
 
def UserAdvancedTabFactory(node, propertiesDialog):
    if (node.Implements('IDSUser')):
        return UserAdvancedTab(propertiesDialog)
    else:
        return None

TabFactories.append(UserAdvancedTabFactory)

class MembersTab:

    def __init__(self, propertiesDialog):
        # Snip out the part of the dialog that we want
        # and add it as a tab
        self.node = propertiesDialog.node

        self.gxml = gtk.glade.XML(GLADE_FILE, 'Members')
        dlg = self.gxml.get_widget('dialog-vbox9')
        dlgcontents = self.gxml.get_widget('dlgcontents')
        dlg.remove(dlgcontents)
        dlg.hide()
        dlg.destroy()
        propertiesDialog.appendPage(dlgcontents, "Members")
        # Connect the glade signals
        function_dict = dict([(f, getattr(self, f)) for f in dir(self)]) 
        self.gxml.signal_autoconnect(function_dict)

        self.add_member_dlg = None

        self.initMemberList()

    def initMemberList(self):

        self.userpb = images['stock_contact']

        self.treeview = self.gxml.get_widget('memberList')
        cellpb = gtk.CellRendererPixbuf()
        cellpb.set_property('xalign', 0.0)
        celltext = gtk.CellRendererText()
        col1 = gtk.TreeViewColumn('Name', cellpb, pixbuf=0)
        col1.pack_start(celltext, False)
        col1.add_attribute(celltext, 'text', 1)
        celltext = gtk.CellRendererText()
        col2 = gtk.TreeViewColumn('Path', celltext, text=2)
        self.treeview.append_column(col1)
        self.treeview.append_column(col2)
        self.refreshMemberList()

    def refreshMemberList(self):
        members = self.node.Members()
        listModel = gtk.ListStore(gtk.gdk.Pixbuf, str, str, object)

        for member in members:
            listModel.append((self.userpb, member.Name(),
                    member.PrettyPath(), member))

        self.treeview.set_model(listModel)

    def OnAddButtonClicked(self, obj, *args):
        gxml = gtk.glade.XML(GLADE_FILE, 'memberOf_AddGroup')
        self.add_member_dlg = gxml.get_widget('memberOf_AddGroup')
        treeview = gxml.get_widget('treeview2')
        description = gxml.get_widget('description')
        description.set_text('<b>Select a user or users to add</b>')
        selection = treeview.get_selection()
        selection.set_mode(gtk.SELECTION_MULTIPLE)
        
        listModel = gtk.ListStore(gtk.gdk.Pixbuf, str, str, object)
        cellpb = gtk.CellRendererPixbuf()
        cellpb.set_property('xalign', 0.0)
        celltext = gtk.CellRendererText()
        col1 = gtk.TreeViewColumn('Name', cellpb, pixbuf=0)
        col1.pack_start(celltext, False)
        col1.add_attribute(celltext, 'text', 1)
        celltext = gtk.CellRendererText()
        col2 = gtk.TreeViewColumn('Path', celltext, text=2)
        treeview.append_column(col1)
        treeview.append_column(col2)
       
        members = self.node.Members(invert=True)
        for member in members:
            listModel.append((self.userpb, member.Name(),
                    member.PrettyPath(), member))
        treeview.set_model(listModel)
        res = self.add_member_dlg.run()
        self.add_member_dlg.hide()
        if (res == gtk.RESPONSE_OK):
            selection = treeview.get_selection()
            treepaths = selection.get_selected_rows()[1]

            for treepath in treepaths:
                iter = listModel.get_iter(treepath)
                userNode = listModel.get_value(iter, 3)
                self.node.Add(userNode)


            if (treepaths):
                self.node.SetInfo()
                self.refreshMemberList()

        self.add_member_dlg.destroy()
        self.add_member_dlg = None
       
    def OnRemoveButtonClicked(self, obj, *args):
        selection = self.treeview.get_selection()
        model, iter = selection.get_selected()
        userNode = model.get_value(iter, 3)
        self.node.Remove(userNode)
        self.node.SetInfo()
        self.refreshMemberList()

    def Apply(self):
        """Apply any changes that have been made through this tab"""

        # the only change that gets made to this object directly is
        # primaryGroup. All other changes are done to the group objects
        # and are handled through the add/remove buttons
        
        return

    def Close(self):
        if self.add_member_dlg:
            self.add_member_dlg.destroy()

def MembersTabFactory(node, propertiesDialog):
    if (node.Implements('IDSGroup') or node.Implements('IDSUserList')):
        return MembersTab(propertiesDialog)

TabFactories.append(MembersTabFactory)
